/*
** BeNES - Nintendo Entertaiment System Emulator for BeOS
**
** * 1999 by makoto yamagata
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <app/Application.h>
#include <interface/MenuItem.h>
#include "BeNESMenu.h"
#include "BeNESMsg.h"

const string BeNESMenu::File = "File";
const string BeNESMenu::Setting = "Setting";
const string BeNESMenu::FrameSkip = "Skip Frames";
const string BeNESMenu::FramesParSec = "Frames Par Sec.";
const string BeNESMenu::Sound = "Sound";
const string BeNESMenu::Input = "Input";

BMenu* BeNESMenu::buildFileMenu()
{
	// Build File Menu
	BMenu* menu = new BMenu(File.c_str());
	BMenuItem* item = NULL;
	item = new BMenuItem("Open...", new BMessage(BENES_OPEN), 'O');
	menu->AddItem(item);
	menu->AddSeparatorItem();
	item = new BMenuItem("ROM Info...", new BMessage(BENES_INFO), 'I'); 
	item->SetEnabled(false);
	menu->AddItem(item);
	menu->AddSeparatorItem();
	item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');
	menu->AddItem(item);

	menu->SetTargetForItems(be_app);
	
	return menu;
}

BMenu* BeNESMenu::buildSettingMenu()
{
	BMenuItem* item = NULL;

	// Build FrameSkip SubMenu
	BMenu* skip = new BMenu(FrameSkip.c_str());
	item = new BMenuItem("0", new BMessage(BENES_FRAMESKIP0));
	skip->AddItem(item);
	item = new BMenuItem("1", new BMessage(BENES_FRAMESKIP1));
	skip->AddItem(item);
	item = new BMenuItem("2", new BMessage(BENES_FRAMESKIP2));
	skip->AddItem(item);
	item = new BMenuItem("3", new BMessage(BENES_FRAMESKIP3));
	skip->AddItem(item);
	item = new BMenuItem("4", new BMessage(BENES_FRAMESKIP4));
	skip->AddItem(item);
	skip->SetTargetForItems(be_app);

	// Build Frames par sec SubMenu
	BMenu* fps = new BMenu(FramesParSec.c_str());
	item = new BMenuItem("30", new BMessage(BENES_FRAMES30));
	fps->AddItem(item);
	item = new BMenuItem("40", new BMessage(BENES_FRAMES40));
	fps->AddItem(item);
	item = new BMenuItem("50", new BMessage(BENES_FRAMES50));
	fps->AddItem(item);
	item = new BMenuItem("60", new BMessage(BENES_FRAMES60));
	fps->AddItem(item);
	item = new BMenuItem("MAX", new BMessage(BENES_FRAMESXX));
	fps->AddItem(item);
	fps->SetTargetForItems(be_app);

	// Build Sound SubMenu
	BMenu* snd = new BMenu(Sound.c_str());
	item = new BMenuItem("Enable", new BMessage(BENES_ENABLESND));
	snd->AddItem(item);
	item = new BMenuItem("Disable", new BMessage(BENES_DISABLESND));
	snd->AddItem(item);
	snd->SetTargetForItems(be_app);
	
	//Build Input Menu
	BMenu* inp = new BMenu(Input.c_str());
	item = new BMenuItem("Joystick", new BMessage(BENES_INPUTJOYSTICK));
	inp->AddItem(item);
	item = new BMenuItem("Gamepad Pro", new BMessage(BENES_INPUTJOYPRO));
	inp->AddItem(item);
	item = new BMenuItem("Keyboard", new BMessage(BENES_INPUTKEYBOARD));
	inp->AddItem(item);
	inp->SetTargetForItems(be_app);
	 
	// Build Setting Menu
	BMenu* menu = new BMenu(Setting.c_str());
	menu->AddItem(skip);
	menu->AddItem(fps);
	menu->AddItem(snd);
	menu->AddItem(inp);
	menu->SetTargetForItems(be_app);
	
	return menu;
}

BMenuBar* BeNESMenu::Build()
{
	BMenuBar* bar = new BMenuBar(BRect(), B_EMPTY_STRING);
	// File Menu
	bar->AddItem(buildFileMenu());
	// Setting Menu
	bar->AddItem(buildSettingMenu());
	return bar;
}
