/*
** BeNES - Nintendo Entertaiment System Emulator for BeOS
**
** * 1999 by makoto yamagata
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef NESMAPPER1_H_DEFINED
#define NESMAPPER1_H_DEFINED

#include "NesAbstructMapper.h"
#include <string>

class NesMapper1: public NesAbstructMapper {
	uchar mRegs[4];
	uchar mCounter;
	uchar mData;
	bool mCart512;
	bool mCart1024;
public:
	void init(NesIOInterface* io) {
		memset(mRegs, 0, sizeof(mRegs));
		mCounter = mData = 0;
		mCart512 = (io->getROM()->sizeofPRGBANKS() == 0x20);
		mCart1024 = (io->getROM()->sizeofPRGBANKS() == 0x40);
	};
	void enter(NesIOInterface* io, ushort address, uchar value);
};

inline void NesMapper1::enter(
	NesIOInterface* io, ushort address, uchar value)
{
	NesPPUInterface* ppu = io->getPPU();
	NesROMInterface* rom = io->getROM();
	NesCPUInterface* cpu = io->getCPU();
	if (value & 0x80) {
		mRegs[0] |= 0x0c;
		mData = mRegs[(address>>13)&0x03];
		mCounter = 5;
	}
	else {
		mData |= (value&0x01)<<(mCounter++);
	}
	if (mCounter == 5) {
		uchar regNum = (address>>13)&0x03;
		mRegs[regNum] = mData;
		mCounter = mData = 0;
		
		// calc prg bank base (fix me)
		int bankbase = 0;
		if (mCart512) {
			bankbase = (mRegs[1] & 0x10)? 0x40000: 0;
		}else if (mCart1024) {
			if (mRegs[0] & 0x10) {
				if (mRegs[1] & 0x10) {
					bankbase += 0x40000;
				}
				if (mRegs[2] & 0x10) {
					bankbase += 0x80000;
				}
			}else {
				bankbase = (mRegs[1] & 0x10)? 0xc0000: 0;
			}
		}
		// mirror settings
		if (!(mRegs[0]&0x02)) {
			ppu->setMirrorType(PPU_MIRROR_ONELOW);
		}else if (mRegs[0]&0x01) {
			ppu->setMirrorType(PPU_MIRROR_HORIZONTAL);
		}else {
			ppu->setMirrorType(PPU_MIRROR_VERTICAL);
		}
		// prg bank set
		if (mRegs[0]&0x08) {
			uchar* mem = rom->getPRGROM(mRegs[3]*0x4000+bankbase);
			if (mRegs[0]&0x04) {
				cpu->memmove(0x8000, mem, 0x4000);
				cpu->memmove(0xc000, 
					rom->getPRGROM(rom->sizeofPRGROM()-NES_PRGBANK_SIZE), 
					0x4000);
			}
			else {
				cpu->memmove(0x8000, rom->getPRGROM(), 0x4000);
				cpu->memmove(0xC000, mem, 0x4000);
			}
		}
		else {
			uchar* mem = rom->getPRGROM((mRegs[3]&0xfe)*0x4000+bankbase);
			cpu->memmove(0x8000, mem, 0x8000);
		}
		// vram bank set
		if (rom->sizeofCHRROM() > 0) {
			if (mRegs[0]&0x10) {
				ppu->memmove(0x0000, rom->getCHRROM(mRegs[1]*0x1000), 0x1000);
				ppu->memmove(0x1000, rom->getCHRROM(mRegs[2]*0x1000), 0x1000);
			}else {
				ppu->memmove(0x0000, 
					rom->getCHRROM((mRegs[1]&0xfe)*0x1000), 0x2000);
			}
		}

	}
}

#endif
