/*
** BeNES - Nintendo Entertaiment System Emulator for BeOS
**
** * 1999 by makoto yamagata
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// 
// The IO Object
//
#ifndef NESIO_H_DEFINED
#define NESIO_H_DEFINED

#include "NesPPU.h"
#include "NesAPU.h"
#include "NesROM.h"
#include "NesJoypad.h"
#include "NesIOInterface.h"

class NesIO: public NesIOInterface {
	NesPPU* mPPU;
	NesAPU* mAPU;
	NesROM* mROM;
	NesCPUInterface* mCPU;
	NesJoypad* mJoy;
public:
	NesIO(NesPPU* ppu, NesAPU* apu):
		mPPU(ppu),
		mAPU(apu),
		mROM(NULL),
		mCPU(NULL),
		mJoy(NULL) { ; }

	// Connect ROM Cartridge.
	void connect(NesROM* rom) { mROM = rom; }
	// Connect JoyPad.
	void connect(NesJoypad* joy) { mJoy = joy; }
	// Connect CPU.
	void connect(NesCPUInterface* cpu) { mCPU = cpu; }
	
	// ROM Interface
	void ROM_Init(uchar* ram);
	void ROM_UnInit(uchar* ram);
	void ROM_Write(ushort address, uchar value)
		{ mROM->write(this, address, value); }
	bool ROM_IsConnected() { return (mROM != NULL); }
	 
	// PPU Interface
	void PPU_ScanLine() {
		mROM->scanLine(this);
		mPPU->scanLine(mCPU);
	}
	// 0x2000
	void PPU_WriteControlA(uchar val) 
		{ mPPU->writeControlA(val); }
	// 0x2001
	void PPU_WriteControlB(uchar val)
		{ mPPU->writeControlB(val); }
	// 0x2002
	uchar PPU_ReadStatus()
		{ return mPPU->readStatus(); }
	// 0x2003
	void PPU_WriteSpriteRamAddress(uchar val)
		{ mPPU->writeSpriteRamAddress(val); }
	// 0x2004
	void PPU_WriteSpriteRam(uchar value)
		{ mPPU->writeSpriteRam(value); }
	uchar PPU_ReadSpriteRam()
		{ return mPPU->readSpriteRam(); }
	// 0x2005
	void PPU_WriteBackgroundScroll(uchar val)
		{ mPPU->writeBackgroundScroll(val); }
	// 0x2006
	void PPU_WritePPURamAddress(uchar val)
		{ mPPU->writePPURamAddress(val); }
	// 0x2007
	void PPU_WritePPURam(uchar value)
		{ mPPU->writePPURam(value); }
	uchar PPU_ReadPPURam()
		{ return mPPU->readPPURam(); }
	//0x4014
	void PPU_SpriteDMA(uchar* ram, uchar value)
		{ mPPU->spriteDMA(&(ram[static_cast<int>(value) * 0x100])); }
	
	// 0x4000-0x4003
	void APU_WriteSq0(int regNum, uchar value)
		{ mAPU->writeSq0(regNum, value); }
	// 0x4004-0x4007
	void APU_WriteSq1(int regNum, uchar value)
		{ mAPU->writeSq1(regNum, value); }
	// 0x4008-0x400B
	void APU_WriteTri(int regNum, uchar value)
		{ mAPU->writeTri(regNum, value); }
	// 0x400C-0x400F
	void APU_WriteNiz(int regNum, uchar value)
		{ mAPU->writeNiz(regNum, value); }
	// 0x4015
	void APU_Write4015(uchar value)
		{ mAPU->write4015(value); }
	// 0x4015
	uchar APU_Read4015()
		{ return mAPU->read4015(); }
	void APU_Reset()
		{ mAPU->reset(); }
	// APU status change
	void APU_StatusChange()
		{ mAPU->statusChange(); }
	// Joypad Interface
	uchar JOY_Read() { return mJoy->read(); }
	void JOY_Reset() { mJoy->reset(); }
public:
	virtual NesCPUInterface* getCPU() { return mCPU; }
	virtual NesPPUInterface* getPPU() { return mPPU; }
	virtual NesROMInterface* getROM() { return mROM; }
};
#endif;
