/*
** BeNES - Nintendo Entertaiment System Emulator for BeOS
**
** * 1999 by makoto yamagata
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef NESPPURAM_H_DEFINED
#define NESPPURAM_H_DEFINED

#include "NesTypes.h"
#include <string>

class NesPPURam {
	uchar mPatternTable[0x2000];
	uchar mNameTable[0x800];
	uchar* m0x2000;
	uchar* m0x2400;
	uchar* m0x2800;
	uchar* m0x2C00;
	uchar mPalette[0x1000];
public:
	NesPPURam() {
		init();
	}
	void init() {
		memset(mPatternTable, 0, sizeof(mPatternTable));
		memset(mNameTable, 0, sizeof(mNameTable));
		memset(mPalette, 0, sizeof(mPalette));
		mirrorHorizontal();
	}
	void mirrorHorizontal() {
		m0x2000 = &(mNameTable[0x000]);
		m0x2400 = &(mNameTable[0x000]);
		m0x2800 = &(mNameTable[0x400]);
		m0x2C00 = &(mNameTable[0x400]);
	}
	void mirrorVertical() {
		m0x2000 = &(mNameTable[0x000]);
		m0x2400 = &(mNameTable[0x400]);
		m0x2800 = &(mNameTable[0x000]);
		m0x2C00 = &(mNameTable[0x400]);
	}
	void mirrorOneLow() {
		m0x2000 = &(mNameTable[0x000]);
		m0x2400 = &(mNameTable[0x000]);
		m0x2800 = &(mNameTable[0x000]);
		m0x2C00 = &(mNameTable[0x000]);
	}
	void mirrorOneHigh() {
		m0x2000 = &(mNameTable[0x400]);
		m0x2400 = &(mNameTable[0x400]);
		m0x2800 = &(mNameTable[0x400]);
		m0x2C00 = &(mNameTable[0x400]);
	}
	uchar& operator[](int address) {
		if (address < 0x2000) {
			return mPatternTable[address];
		}
		else if (address < 0x2400) {
			return m0x2000[address&0x3ff];
		}
		else if (address < 0x2800) {
			return m0x2400[address&0x3ff];
		}
		else if (address < 0x2C00) {
			return m0x2800[address&0x3ff];
		}
		else if (address < 0x3000) {
			return m0x2C00[address&0x3ff];
		}
		else {
			return mPalette[address&0xfff];
		}
	}
};

#endif
