/*
** BeNES - Nintendo Entertaiment System Emulator for BeOS
**
** * 1999 by makoto yamagata
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
//
// ROM IO
//
#ifndef NESROM_H_DEFINED
#define NESROM_H_DEFINED

#include <fstream>
#include <string>
#include "NesTypes.h"
#include "NesAbstructMapper.h"
#include "NesROMInterface.h"

struct NesRomHeader {
	uchar id[4];
	uchar prg_banks;
	uchar chr_banks;
	uchar rom_info1;
	uchar rom_info2;
	uchar reserve[8];
};

class NesROM: public NesROMInterface {
	NesRomHeader mHeader;
	uchar* mPrgrom;
	size_t mPrgromSize;
	uchar* mChrrom;
	size_t mChrromSize;
	uchar* mTrainer;
	uchar* mSaveRam;
	string mSaveName;
	NesAbstructMapper* mMapper;
public:
	NesROM(): 
		mPrgrom(NULL), 
		mChrrom(NULL), 
		mTrainer(NULL), 
		mSaveRam(NULL),
		mMapper(NULL) { ; }
	~NesROM();
	
	void loadRomImage(const char* name);
	
	// 初期化
	void init(NesIOInterface* io);
	// 後処理
	void uninit(NesIOInterface* io);
	// ROMへ書込み
	void write(NesIOInterface* io, ushort address, uchar value)
		{ mMapper->enter(io, address, value); }
	// Scanline
	void scanLine(NesIOInterface* io)
		{ mMapper->scanLine(io); }
		
	// マッパー番号の取得
	int getMapperNumber() {
		int mapper = static_cast<int>((mHeader.rom_info1>>4)&0x0f);
		if ((mHeader.rom_info2&0x0f) == 0) {
			mapper |= (mHeader.rom_info2&0xf0);
		}
		return mapper;
	}
	// ミラーリングタイプの取得
	bool isVerticalMirror() {
		return ((mHeader.rom_info1&0x01) != 0);
	}
	// SaveRAMは持ってるか？
	bool hasSaveRam() {
		return ((mHeader.rom_info1&0x02) != 0);
	}
	// Trainerモードを持ってるか？
	bool hasTrainer() {
		return ((mHeader.rom_info1&0x04) != 0);
	}
public:
	virtual size_t sizeofPRGBANKS() {
		return static_cast<int>(mHeader.prg_banks);
	}
	virtual size_t sizeofCHRBANKS() {
		return static_cast<int>(mHeader.chr_banks);
	}
	virtual size_t sizeofPRGROM() {
		return mPrgromSize;
	}
	virtual uchar* getPRGROM(size_t address=0L) {
		return (mPrgrom+(address & (mPrgromSize-1)));
	}
	virtual size_t sizeofCHRROM() {
		return mChrromSize;
	}
	virtual uchar* getCHRROM(size_t address=0L) {
		if (mChrrom) { return (mChrrom+address); }
		else { return (mPrgrom+(address & (mChrromSize-1))); }
	}
	virtual uchar* getSAVERAM() {
		return mSaveRam;
	}
	virtual uchar* getTRAINER() {
		return mTrainer;
	}
private:
	uchar* readTrainer(ifstream& ifs);
	uchar* readPrgrom(ifstream& ifs, int* prg_size);
	uchar* readChrrom(ifstream& ifs, int* chr_size);
	uchar* readSaveRam(const char* name);
};

#endif;
